"""Integration tests for EasyName"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


class EasynameProviderTests(TestCase, IntegrationTests):
    """TestCase for EasyName"""
    provider_name = 'easyname'
    domain = 'lexicontest.astzweig.de'

    def _filter_post_data_parameters(self):
        return ['username', 'password']

    def _filter_headers(self):
        return ['Cookie']

    def _test_fallback_fn(self):
        return lambda x: 'placeholder_' + x if x != 'priority' else ''



"""General pytest configuration for lexicon tests."""
import pytest

from lexicon import discovery


def pytest_addoption(parser):
    """Standard pytest hook invoked to add options to pytest CLI"""
    parser.addoption('--xfail-providers-with-missing-deps', action='store_true',
                     help='Skip tests on providers with optional dependencies')


def pytest_runtest_setup(item):
    """Standard pytest hook invoked before each test execution"""
    try:
        skip_providers_with_optdeps = getattr(item.config.option,
                                              'xfail_providers_with_missing_deps')
    except AttributeError:
        pass
    else:
        if skip_providers_with_optdeps:
            providers = discovery.find_providers()
            skip = [available for provider, available in providers.items()
                    if provider in item.parent.name.lower()]
            if skip and not skip[0]:
                pytest.xfail('Test expected to fail because --skip-providers-with-missing-deps '
                             'is set and provider has missing required dependencies.')



"""Module provider for Cloudflare"""
from __future__ import absolute_import
import json
import logging

import requests
from lexicon.providers.base import Provider as BaseProvider


LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['cloudflare.com']


def provider_parser(subparser):
    """Return the parser for this provider"""
    subparser.add_argument(
        "--auth-username", help="specify email address for authentication")
    subparser.add_argument(
        "--auth-token", help="specify token for authentication")


class Provider(BaseProvider):
    """Provider class for Cloudflare"""
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None
        self.api_endpoint = 'https://api.cloudflare.com/client/v4'

    def _authenticate(self):

        payload = self._get('/zones', {
            'name': self.domain,
            'status': 'active'
        })

        if not payload['result']:
            raise Exception('No domain found')
        if len(payload['result']) > 1:
            raise Exception('Too many domains found. This should not happen')

        self.domain_id = payload['result'][0]['id']

    # Create record. If record already exists with the same content, do nothing'

    def _create_record(self, rtype, name, content):
        data = {'type': rtype, 'name': self._full_name(
            name), 'content': content}
        if self._get_lexicon_option('ttl'):
            data['ttl'] = self._get_lexicon_option('ttl')

        payload = {'success': True}
        try:
            payload = self._post(
                '/zones/{0}/dns_records'.format(self.domain_id), data)
        except requests.exceptions.HTTPError as err:
            already_exists = next((True for error in err.response.json()[
                'errors'] if error['code'] == 81057), False)
            if not already_exists:
                raise

        LOGGER.debug('create_record: %s', payload['success'])
        return payload['success']

    # List all records. Return an empty list if no records found
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is received.
    def _list_records(self, rtype=None, name=None, content=None):
        filter_obj = {'per_page': 100}
        if rtype:
            filter_obj['type'] = rtype
        if name:
            filter_obj['name'] = self._full_name(name)
        if content:
            filter_obj['content'] = content

        payload = self._get(
            '/zones/{0}/dns_records'.format(self.domain_id), filter_obj)

        records = []
        for record in payload['result']:
            processed_record = {
                'type': record['type'],
                'name': record['name'],
                'ttl': record['ttl'],
                'content': record['content'],
                'id': record['id']
            }
            records.append(processed_record)

        LOGGER.debug('list_records: %s', records)
        return records

    # Create or update a record.
    def _update_record(self, identifier, rtype=None, name=None, content=None):

        data = {}
        if rtype:
            data['type'] = rtype
        if name:
            data['name'] = self._full_name(name)
        if content:
            data['content'] = content
        if self._get_lexicon_option('ttl'):
            data['ttl'] = self._get_lexicon_option('ttl')

        payload = self._put(
            '/zones/{0}/dns_records/{1}'.format(self.domain_id, identifier), data)

        LOGGER.debug('update_record: %s', payload['success'])
        return payload['success']

    # Delete an existing record.
    # If record does not exist, do nothing.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        delete_record_id = []
        if not identifier:
            records = self._list_records(rtype, name, content)
            delete_record_id = [record['id'] for record in records]
        else:
            delete_record_id.append(identifier)

        LOGGER.debug('delete_records: %s', delete_record_id)

        for record_id in delete_record_id:
            self._delete(
                '/zones/{0}/dns_records/{1}'.format(self.domain_id, record_id))

        LOGGER.debug('delete_record: %s', True)
        return True

    # Helpers
    def _request(self, action='GET', url='/', data=None, query_params=None):
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}
        response = requests.request(action, self.api_endpoint + url, params=query_params,
                                    data=json.dumps(data),
                                    headers={
                                        'X-Auth-Email': self._get_provider_option('auth_username'),
                                        'X-Auth-Key': self._get_provider_option('auth_token'),
                                        'Content-Type': 'application/json'
                                    })
        # if the request fails for any reason, throw an error.
        response.raise_for_status()
        return response.json()



"""Module provider for Hover"""
from __future__ import absolute_import

import json
import logging
import requests

from lexicon.providers.base import Provider as BaseProvider

LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['hover.com']


def provider_parser(subparser):
    """Return the parser for this provider"""
    subparser.add_argument(
        "--auth-username", help="specify username for authentication")
    subparser.add_argument(
        "--auth-password", help="specify password for authentication")


class Provider(BaseProvider):
    """Provider class for Hover"""

    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None
        self.api_endpoint = 'https://www.hover.com/api'
        self.cookies = {}

    def _authenticate(self):
        # Getting required cookies "hover_session" and "hoverauth"
        response = requests.get("https://www.hover.com/signin")
        self.cookies["hover_session"] = response.cookies['hover_session']

        payload = {"username": self._get_provider_option('auth_username'),
                   "password": self._get_provider_option('auth_password')}
        response = requests.post("https://www.hover.com/signin/auth.json",
                                 json=payload,
                                 cookies=self.cookies)
        response.raise_for_status()

        if "hoverauth" not in response.cookies:
            raise Exception("Unexpected auth response")
        self.cookies["hoverauth"] = response.cookies["hoverauth"]

        # Make sure domain exists
        # domain is stored in self.domain from BaseProvider

        domains = self._list_domains()
        for domain in domains:
            if domain['name'] == self.domain:
                self.domain_id = domain['id']

        if self.domain_id is None:
            raise Exception('Domain {} not found'.format(self.domain))

    def _list_domains(self):
        response = self._get('/domains')

        domains = []
        for domain in response['domains']:
            processed_domain = {
                'name': domain['domain_name'],
                'id': domain['id'],
                'active': (domain['status'] == "active")
            }
            domains.append(processed_domain)

        LOGGER.debug('list_domains: %s', domains)
        return domains

    # List all records. Return an empty list if no records found
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is received.
    def _list_records(self, rtype=None, name=None, content=None):
        payload = self._get('/domains/{0}/dns'.format(self.domain_id))

        # payload['domains'] should be a list of len 1
        try:
            raw_records = payload['domains'][0]['entries']
        except (KeyError, IndexError):
            raise Exception("Unexpected response")

        processed_records = []
        for record in raw_records:
            processed_record = {
                'type': record['type'],
                'name': self._full_name(record['name']),
                'ttl': record['ttl'],
                'content': record['content'],
                'id': record['id']
            }
            processed_records.append(processed_record)

        if rtype:
            processed_records = [record for record in processed_records if record['type'] == rtype]
        if name:
            name = self._relative_name(name)
            processed_records = [record for record in processed_records if name in record['name']]
        if content:
            processed_records = [record for record in processed_records
                                 if record['content'].lower() == content.lower()]

        LOGGER.debug('list_records: %s', processed_records)
        return processed_records

    def _create_record(self, rtype, name, content):
        name = self._relative_name(name)
        records = self._list_records(rtype, name, content)
        if records:
            LOGGER.debug('not creating duplicate record: %s', records[0])
            return True

        record = {"name": name,
                  "type": rtype,
                  "content": content}
        if self._get_lexicon_option('ttl'):
            record['ttl'] = self._get_lexicon_option('ttl')

        LOGGER.debug('create_record: %s', record)
        payload = self._post("/domains/{0}/dns".format(self.domain_id), record)
        return payload['succeeded']

    # Update a record. Hover cannot update name so we delete and recreate.
    def _update_record(self, identifier, rtype=None, name=None, content=None):
        if identifier:
            records = self._list_records()
            records = [r for r in records if r['id'] == identifier]
        else:
            records = self._list_records(rtype, name, None)

        if not records:
            raise Exception("Record not found")
        if len(records) > 1:
            raise Exception("Record not unique")
        orig_record = records[0]
        orig_id = orig_record['id']

        new_rtype = rtype if rtype else orig_record['type']
        new_name = name if name else orig_record['name']
        new_content = content if content else orig_record['content']

        self._delete_record(orig_id)
        return self._create_record(new_rtype, new_name, new_content)

    # Delete an existing record.
    # If record does not exist, do nothing.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        delete_record_ids = []
        if not identifier:
            records = self._list_records(rtype, name, content)
            delete_record_ids = [record['id'] for record in records]
        else:
            delete_record_ids.append(identifier)

        LOGGER.debug('delete_records: %s', delete_record_ids)

        for record_id in delete_record_ids:
            self._delete("/dns/{0}".format(record_id))
            LOGGER.debug('delete_record: %s', record_id)
        return True

    # Helpers
    def _request(self, action='GET', url='/', data=None, query_params=None):
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}
        response = requests.request(action, self.api_endpoint + url,
                                    params=query_params,
                                    data=json.dumps(data),
                                    cookies=self.cookies,
                                    headers={'Content-Type': 'application/json'})

        # if the request fails for any reason, throw an error.
        response.raise_for_status()
        try:
            return response.json()
        except ValueError:  # response is not json
            raise Exception("Did not get JSON response.")




"""
Lexicon Plesk Provider

Author: Jens Reimann, 2018

API Docs: https://docs.plesk.com/en-US/onyx/api-rpc
"""
from __future__ import absolute_import
import logging
from collections import OrderedDict

import requests
from lexicon.providers.base import Provider as BaseProvider


try:
    import xmltodict  # optional dependency
except ImportError:
    pass


LOGGER = logging.getLogger(__name__)

PLEX_URL_SUFFIX = "/enterprise/control/agent.php"

NAMESERVER_DOMAINS = []


def provider_parser(subparser):
    """Configure provider parser for Plesk"""
    subparser.add_argument(
        "--auth-username", help="specify username for authentication")
    subparser.add_argument(
        "--auth-password", help="specify password for authentication")
    subparser.add_argument(
        '--plesk-server', help="specify URL to the Plesk Web UI, including the port")


class Provider(BaseProvider):
    """Provider class for Plesk"""
    def __init__(self, config):
        super(Provider, self).__init__(config)

        self.api_endpoint = self._get_provider_option('plesk_server')

        if self.api_endpoint.endswith('/'):
            self.api_endpoint = self.api_endpoint[:-1]

        if not self.api_endpoint.endswith(PLEX_URL_SUFFIX):
            self.api_endpoint += PLEX_URL_SUFFIX

        self.site_name = self.domain
        assert self.site_name is not None

        self.domain_id = None

        self.username = self._get_provider_option('auth_username')
        assert self.username is not None

        self.password = self._get_provider_option('auth_password')
        assert self.password is not None

    def __simple_request(self, rtype, operation, req):

        response = self.__plesk_request({
            rtype: {
                operation: req
            }
        })[rtype][operation]

        result = response["result"]

        if isinstance(result, list):
            for record in result:
                if record["status"] == "error":
                    raise Exception(
                        "API returned at least one error: %s" % record["errtext"])
        elif response["result"]["status"] == "error":
            errcode = response["result"]["errcode"]
            errtext = response["result"]["errtext"]
            raise Exception("API returned error: %s (%s)" % (errcode, errtext))

        return response

    def __plesk_request(self, request):

        headers = {}
        headers["Content-type"] = "text/xml"
        headers["HTTP_PRETTY_PRINT"] = "TRUE"
        headers["HTTP_AUTH_LOGIN"] = self.username
        headers["HTTP_AUTH_PASSWD"] = self.password

        xml = xmltodict.unparse({
            "packet": request
        }, pretty=True)

        LOGGER.debug("Request: %s", xml)

        response = requests.post(self.api_endpoint, headers=headers,
                                 data=xml, auth=(self.username, self.password))

        data = response.text

        LOGGER.debug("Response: %s", data)
        result = xmltodict.parse(data)
        return result["packet"]

    def __find_site(self):
        return self.__simple_request('site', 'get', OrderedDict([
            ('filter', {'name': self.site_name}),
            ('dataset', {})
        ]))["result"]["id"]

    def _authenticate(self):
        self.domain_id = self.__find_site()

        if self.domain_id is None:
            raise Exception('Domain not found')

    def _create_record(self, rtype, name, content):
        return self.__create_entry(rtype, name, content, None)

    def _list_records(self, rtype=None, name=None, content=None):
        entries = self.__find_dns_entries(rtype, name, content)
        LOGGER.debug("list_records: %s", entries)
        return entries

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        if identifier is None:
            entries = self.__find_dns_entries(rtype, name, None)
            LOGGER.debug("Entries found: %s", entries)

            if not entries:
                raise Exception("No entry found for updating")

            identifier = entries[0]["id"]
            entry = self.__get_dns_entry(identifier)

            ids = []
            for an_entry in entries:
                ids.append(an_entry["id"])

            self.__delete_dns_records_by_id(ids)

        else:

            entry = self.__get_dns_entry(identifier)
            self.__delete_dns_records_by_id([identifier])

        assert entry is not None

        LOGGER.debug("Updating: %s", entry)

        if rtype:
            entry["type"] = rtype
        if name:
            entry["host"] = name
        if content:
            entry["value"] = content

        return self.__create_entry(entry["type"], entry["host"], entry["value"], entry["opt"])

    def __create_entry(self, rtype, host, value, opt):
        entries = self.__find_dns_entries(rtype, self._fqdn_name(host), value)

        if entries:
            return True  # already exists

        self.__simple_request('dns', 'add_rec', OrderedDict([
            ('site-id', self.domain_id),
            ('type', rtype),
            ('host', self._relative_name(host)),
            ('value', value),
            ('opt', opt)
        ]))

        return True

    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        if identifier:
            self.__delete_dns_records_by_id([identifier])
            return True
        entries = self.__find_dns_entries(
            rtype, self._fqdn_name(name), content)
        ids = []

        for entry in entries:
            ids.append(entry["id"])

        self.__delete_dns_records_by_id(ids)
        return bool(ids)

    def __get_dns_entry(self, identifier):
        return self.__simple_request('dns', 'get_rec', {
            'filter': {
                'id': identifier
            }
        })["result"]["data"]

    def __find_dns_entries(self, rtype=None, host=None, value=None):
        LOGGER.debug("Searching for: %s, %s, %s", rtype, host, value)

        if value and rtype and rtype in ["CNAME"]:
            LOGGER.debug("CNAME transformation")
            value = value.rstrip('.') + "."

        if host:
            host = self._fqdn_name(host)

        result = self.__simple_request('dns', 'get_rec', {
            'filter': {
                'site-id': self.domain_id
            }
        })

        entries = []

        for record in result["result"]:

            LOGGER.debug("Record: %s", record)

            if (rtype is not None) and (record["data"]["type"] != rtype):
                LOGGER.debug(
                    "\tType doesn't match - expected: '%s', found: '%s'",
                    rtype, record["data"]["type"])
                continue

            if (host is not None) and (record["data"]["host"] != host):
                LOGGER.debug(
                    "\tHost doesn't match - expected: '%s', found: '%s'",
                    host, record["data"]["host"])
                continue

            if (value is not None) and (record["data"]["value"] != value):
                LOGGER.debug(
                    "\tValue doesn't match - expected: '%s', found: '%s'",
                    value,
                    record["data"]["value"])
                continue

            entry = {
                'id': record["id"],
                'type': record["data"]["type"],
                'name': self._full_name(record["data"]["host"]),
                'ttl': None,
                'options': {}
            }

            if record["data"]["type"] in ["CNAME"]:
                entry['content'] = record["data"]["value"].rstrip('.')
            else:
                entry['content'] = record["data"]["value"]

            if record["data"]["type"] == "MX":
                entry['options']['mx'] = {
                    'priority': int(record["data"]["opt"])
                }

            entries.append(entry)

        return entries

    def __delete_dns_records_by_id(self, ids):
        if not ids:
            return

        req = []
        for i in ids:
            req.append({
                'del_rec': {
                    'filter': {
                        'id': i
                    }
                }
            })

        self.__plesk_request({
            'dns': req
        })

    def _request(self, action='GET', url='/', data=None, query_params=None):
        # Helper _request is not used for Plesk provider
        pass



"""Test for route53 implementation of the interface."""
# pylint: disable=missing-docstring
from unittest import TestCase
from contextlib import contextmanager

import pytest

from lexicon.tests.providers.integration_tests import IntegrationTests
from lexicon.tests.providers.integration_tests import PROVIDER_VCR
from lexicon.tests.providers.integration_tests import EngineOverrideConfigSource


@pytest.mark.skip('Cassettes needs to be regenerated')
class Route53ProviderTests(TestCase, IntegrationTests):
    """Route53 Proivder Tests."""

    provider_name = 'route53'
    domain = 'capsulecd.com'

    def _filter_headers(self):
        """Sensitive headers to be filtered."""
        return ['Authorization']

    def test_provider_authenticate_private_zone_only(self):
        with self._use_vcr('IntegrationTests/test_provider_authenticate.yaml'):
            provider = self._build_provider_with_overrides({'private_zone': 'true'})
            with pytest.raises(Exception):
                provider.authenticate()

    def test_provider_authenticate_private_zone_false(self):
        with self._use_vcr('IntegrationTests/test_provider_authenticate.yaml'):
            provider = self._build_provider_with_overrides({'private_zone': 'false'})
            provider.authenticate()
            assert provider.domain_id is not None

    def _build_provider_with_overrides(self, overrides):
        config = self._test_config()
        config.add_config_source(EngineOverrideConfigSource(overrides), 0)
        return self.provider_module.Provider(config)

    @contextmanager
    def _use_vcr(self, path):
        with PROVIDER_VCR.use_cassette(
                self._cassette_path(path),
                filter_headers=self._filter_headers(),
                before_record_response=self._filter_response,
                filter_query_parameters=self._filter_query_parameters(),
                filter_post_data_parameters=self._filter_post_data_parameters()):
            yield

    @pytest.mark.skip(reason="route 53 dns records don't have ids")
    def test_provider_when_calling_delete_record_by_identifier_should_remove_record(self):
        return

    # TODO: the following skipped suite and fixtures should be enabled
    @pytest.mark.skip(reason="new test, missing recording")
    def test_provider_when_calling_update_record_should_modify_record_name_specified(self):
        return

    @pytest.fixture(autouse=True)
    def _skip_suite(self, request):  # pylint: disable=no-self-use
        if request.node.get_closest_marker('ext_suite_1'):
            pytest.skip('Skipping extended suite')



"""Integration tests for DNSSimple"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from define_tests.TheTests
class DnsimpleProviderTests(TestCase, IntegrationTests):
    """TestCase for DNSSimple"""
    provider_name = 'dnsimple'
    domain = 'lexicontest.us'

    def _test_parameters_overrides(self):
        return {
            'api_endpoint': 'https://api.sandbox.dnsimple.com/v2',
            'region': 'global'
        }

    def _filter_headers(self):
        return ['Authorization', 'set-cookie', 'X-Dnsimple-OTP']



from lexicon.config import ConfigResolver
from lexicon.client import Client

lexicon_config = {
    "provider_name" : "cloudflare", # lexicon shortname for provider, see providers directory for available proviers
    "action": "list", # create, list, update, delete
    "domain": "capsulecd.com", # domain name
    "type": "CNAME", # specify a type for record filtering, case sensitive in some cases.
    "cloudflare": {
        # cloudflare(provider) specific configuration goes here.
        # if .with_env() is not used, all credentials required for authention must be specified here.
    }
}

config = ConfigResolver()
config.with_env().with_dict(dict_object=lexicon_config)
client = Client(config)
results = client.execute()
print results


"""Module provider for INWX"""
from __future__ import absolute_import
import logging

from lexicon.providers.base import Provider as BaseProvider


try:
    import xmlrpclib
except ImportError:
    import xmlrpc.client as xmlrpclib

LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['inwx.com']


def provider_parser(subparser):
    """Configure provider parser for INWX"""
    subparser.add_argument("--auth-username",
                           help="specify username for authentication")
    subparser.add_argument("--auth-password",
                           help="specify password for authentication")


class Provider(BaseProvider):
    """
    INWX offers a free testing system on https://ote.inwx.com
    see https://www.inwx.de/en/offer/api for details about ote and the api
    """
    def __init__(self, config):
        """
        :param config: command line options
        """
        super(Provider, self).__init__(config)

        self._auth = {'user': self._get_provider_option('auth_username'),
                      'pass': self._get_provider_option('auth_password')}
        self._domain = self.domain.lower()
        self.domain_id = None

        endpoint = self._get_provider_option(
            'endpoint') or 'https://api.domrobot.com/xmlrpc/'

        self._api = xmlrpclib.ServerProxy(endpoint, allow_none=True)

    def _validate_response(self, response, message, exclude_code=None):  # pylint: disable=no-self-use
        """
        validate an api server response

        :param dict response: server response to check
        :param str message: error message to raise
        :param int exclude_code: error codes to exclude from errorhandling
        :return:
        ":raises Exception: on error
        """
        if 'code' in response and response['code'] >= 2000:
            if exclude_code is not None and response['code'] == exclude_code:
                return

            raise Exception("{0}: {1} ({2})".format(
                message, response['msg'], response['code']))

    # Make any request to validate credentials
    def _authenticate(self):
        """
        run any request against the API just to make sure the credentials
        are valid

        :return bool: success status
        :raises Exception: on error
        """
        opts = {'domain': self._domain}
        opts.update(self._auth)
        response = self._api.domain.info(opts)
        self._validate_response(
            response=response, message='Failed to authenticate')

        # set to fake id to pass tests, inwx doesn't work on domain id but
        # uses domain names for identification
        self.domain_id = 1

        return True

    def _create_record(self, rtype, name, content):
        """
        create a record
        does nothing if the record already exists

        :param str rtype: type of record
        :param str name: name of record
        :param mixed content: value of record
        :return bool: success status
        :raises Exception: on error
        """
        opts = {'domain': self._domain, 'type': rtype.upper(),
                'name': self._full_name(name), 'content': content}
        if self._get_lexicon_option('ttl'):
            opts['ttl'] = self._get_lexicon_option('ttl')
        opts.update(self._auth)

        response = self._api.nameserver.createRecord(opts)
        self._validate_response(
            response=response, message='Failed to create record',
            exclude_code=2302)

        return True

    def _list_records(self, rtype=None, name=None, content=None):
        """
        list all records

        :param str rtype: type of record
        :param str name: name of record
        :param mixed content: value of record
        :return list: list of found records
        :raises Exception: on error
        """
        opts = {'domain': self._domain}
        if rtype is not None:
            opts['type'] = rtype.upper()
        if name is not None:
            opts['name'] = self._full_name(name)
        if content is not None:
            opts['content'] = content
        opts.update(self._auth)

        response = self._api.nameserver.info(opts)
        self._validate_response(
            response=response, message='Failed to get records')

        records = []
        if 'record' in response['resData']:
            for record in response['resData']['record']:
                processed_record = {
                    'type': record['type'],
                    'name': record['name'],
                    'ttl': record['ttl'],
                    'content': record['content'],
                    'id': record['id']
                }
                records.append(processed_record)

        return records

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        """
        update a record

        :param int identifier: identifier of record to update
        :param str rtype: type of record
        :param str name: name of record
        :param mixed content: value of record
        :return bool: success status
        :raises Exception: on error
        """
        record_ids = []
        if not identifier:
            records = self._list_records(rtype, name)
            record_ids = [record['id'] for record in records]
        else:
            record_ids.append(identifier)

        for an_identifier in record_ids:
            opts = {'id': an_identifier}
            if rtype is not None:
                opts['type'] = rtype.upper()
            if name is not None:
                opts['name'] = self._full_name(name)
            if content is not None:
                opts['content'] = content
            opts.update(self._auth)

            response = self._api.nameserver.updateRecord(opts)
            self._validate_response(
                response=response, message='Failed to update record',
                exclude_code=2302)

        return True

    def _delete_record(self, identifier=None, rtype=None, name=None,
                       content=None):
        """
        delete a record
        filter selection to delete by identifier or rtype/name/content

        :param int identifier: identifier of record to update
        :param str rtype: rtype of record
        :param str name: name of record
        :param mixed content: value of record
        :return bool: success status
        :raises Exception: on error
        """
        record_ids = []
        if not identifier:
            records = self._list_records(rtype, name, content)
            record_ids = [record['id'] for record in records]
        else:
            record_ids.append(identifier)

        for record_id in record_ids:
            opts = {'id': record_id}
            opts.update(self._auth)
            response = self._api.nameserver.deleteRecord(opts)
            self._validate_response(
                response=response, message='Failed to update record')

        return True

    def _request(self, action='GET', url='/', data=None, query_params=None):
        # Helper _request is not used for INWX provider.
        pass



"""Provide support to Lexicon for Subreg.cz DNS changes."""
from __future__ import absolute_import
from builtins import staticmethod

from lexicon.providers.gransy import Provider as GransyProvider, gransy_provider_parser

NAMESERVER_DOMAINS = []

def provider_parser(subparser):
    """Configure provider parser"""
    gransy_provider_parser(subparser)

    subparser.description = "Compatibility proxy for Gransy site subreg.cz. The Subreg " \
        "provider is deprecated, use Gransy provider instead."

class Provider(GransyProvider):
    """Provider class for Subreg"""

    @staticmethod
    def _raise_error(major, minor, message):
        raise SubregError(major, minor, message)

class SubregError(Exception):
    """Specific error for Subreg provider"""
    def __init__(self, major, minor, message):
        self.major = int(major)
        self.minor = int(minor)
        self.message = message
        super(SubregError, self).__init__()

    def __str__(self):
        return 'Major: {} Minor: {} Message: {}'.format(self.major, self.minor, self.message)



"""Integration tests for RcodeZero"""
from unittest import TestCase
from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class RcodezeroProviderTests(TestCase, IntegrationTests):
    """TestCase for RcodeZero"""
    provider_name = 'rcodezero'
    domain = 'lexicon-test.at'

    def _filter_headers(self):
        return ['Authorization']
